'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createProjectSync = createProjectSync;
exports.configPackageSync = configPackageSync;
exports.copyTemplates = copyTemplates;
exports.installPackage = installPackage;
exports.versionControlSync = versionControlSync;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _ora = require('ora');

var _ora2 = _interopRequireDefault(_ora);

var _ncp = require('ncp');

var _temp = require('temp');

var _temp2 = _interopRequireDefault(_temp);

var _shelljs = require('shelljs');

var _logger = require('./logger');

var _logger2 = _interopRequireDefault(_logger);

var _utils = require('./utils');

var _config = require('./config');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createProjectSync(projectDir) {
  var spinner = (0, _ora2.default)('1. Create project in ' + projectDir).start();

  try {
    var mkdirResult = (0, _shelljs.mkdir)(projectDir);

    if (mkdirResult.code !== 0) {
      throw new Error(mkdirResult.stderr);
    }

    var cdResult = (0, _shelljs.cd)(projectDir);

    if (cdResult.code !== 0) {
      throw new Error(cdResult.stderr);
    }

    spinner.succeed();
  } catch (error) {
    _logger2.default.error(error.message);
    spinner.fail();
    (0, _shelljs.exit)(1);
  }
}

function configPackageSync() {
  var spinner = (0, _ora2.default)('2. Configurate package.json').start();

  try {
    (0, _shelljs.exec)('npm init --force', { silent: true });

    /* eslint-disable global-require, no-dynamic-require */
    var packageJson = require((0, _utils.resolvePath)('package.json'));
    /* eslint-enable */

    var overridedConfig = Object.assign(packageJson, _config.PACKAGE_JSON);
    var stringifiedConfig = JSON.stringify(overridedConfig, null, 2);

    _fs2.default.writeFileSync((0, _utils.resolvePath)('package.json'), stringifiedConfig);

    spinner.succeed();
  } catch (error) {
    _logger2.default.error(error.message);
    spinner.fail();
    (0, _shelljs.exit)(1);
  }
}

function copyTemplates(projectDir) {
  return new Promise(function (resolve, reject) {
    var spinner = (0, _ora2.default)('3. Copy template files to src folder').start();
    var tempPath = _temp2.default.mkdirSync('templates');

    (0, _shelljs.exec)('git clone --quiet ' + _config.TEMPLATE_REPO + ' ' + tempPath);

    (0, _ncp.ncp)(_path2.default.join(tempPath, 'src'), _path2.default.join(projectDir, 'src'), function (error) {
      if (error) {
        _logger2.default.error(error);
        spinner.fail();
        reject(error);
        return;
      }

      _fsExtra2.default.copySync(_path2.default.join(tempPath, '.gitignore'), _path2.default.join(projectDir, '.gitignore'));

      _fsExtra2.default.copySync(_path2.default.join(tempPath, 'Procfile'), _path2.default.join(projectDir, 'Procfile'));

      _fsExtra2.default.copySync(_path2.default.join(tempPath, 'webpack.config.babel.js'), _path2.default.join(projectDir, 'webpack.config.babel.js'));

      spinner.succeed();
      resolve();
    });
  });
}

function installPackage() {
  var spinner = (0, _ora2.default)('4. Install package dependencies').start();

  var depPromises = _config.DEPENDENCIES.map(function (depSettings) {
    return new Promise(function (resolve) {
      (0, _shelljs.exec)('yarn add ' + (depSettings.dev ? '--dev' : '') + ' ' + depSettings.packages.join(' '), function () {
        resolve();
      });
    });
  });

  return Promise.all(depPromises).then(function (result) {
    spinner.succeed();
    return result;
  }).catch(function (errors) {
    spinner.fail();
    throw errors;
  });
}

function versionControlSync() {
  var commands = ['git init', 'git add .', 'git commit -m "Initial commit"'];

  var spinner = (0, _ora2.default)('5. Setup version control').start();

  try {
    commands.forEach(function (command) {
      var result = (0, _shelljs.exec)(command, { silent: true });

      if (result.code !== 0) {
        throw new Error(result.stderr);
      }
    });

    spinner.succeed();
  } catch (error) {
    _logger2.default.error(error.message);
    spinner.fail();
    (0, _shelljs.exit)(1);
  }
}
//# sourceMappingURL=setup.js.map