import test from 'ava';

import temp from 'temp';
import isObject from 'lodash/isObject';

import {
  exec,
  ls,
} from 'shelljs';

import {
  configPackageSync,
  versionControlSync,
  copyTemplates,
} from '../src/setup';

temp.track();

test('should copy template successfully', async t => {
  // Create temp path for testing
  const path = temp.mkdirSync('project');

  await copyTemplates(path);
  const { stdout: fileList } = ls(path);

  t.is(fileList.includes('src'), true);
  t.is(fileList.includes('webpack.config.babel.js'), true);
});

// it is not expected to test on remote server like CI since there is no git account configurated
test('should contain initial commit with version control', t => {
  const { stdout: gitUserName } = exec('git config user.name');

  const gitConfigurated = gitUserName.trim() === 'George Jor';

  if (!gitConfigurated) {
    t.pass();
    return;
  }

  // Create temp path for testing
  const path = temp.mkdirSync('project');
  process.chdir(path);

  // Create a dummy file for `git add .`
  exec('echo "" > dummy.txt');

  versionControlSync();

  const { stdout: gitStatus } = exec('git status');
  const { stdout: gitLog } = exec('git log --pretty=oneline --abbrev-commit -1');

  t.is(gitStatus.includes('On branch master'), true);
  t.is(gitLog.includes('Initial commit'), true);

  t.pass();
});

test('should create and modify package.json successfully', (t) => {
  // Create temp path for testing
  const path = temp.mkdirSync('project');
  process.chdir(path);

  configPackageSync();

  /* eslint-disable global-require */
  const targetFile = require(`${path}/package.json`);
  /* eslint-enable */

  t.is(isObject(targetFile), true);

  t.deepEqual(targetFile.scripts, {
    lint: 'eslint src',
    
    start: 'per-env',
    'start:development': 'webpack-dev-server',
    'start:production': 'npm run build && pushstate-server',
  });

  t.is(targetFile.license, 'UNLICENSED');

  t.deepEqual(targetFile.eslintConfig, {
    extends: 'eslint-config-airbnb',
    parser: 'babel-eslint',
    env: {
      browser: true,
      node: true,
    },
    globals: {
      React: true,
      classnames: true,
      $: true,
    },
  });

  t.deepEqual(targetFile.babel, {
    presets: ['latest', 'react', 'react-optimize'],
  });

  t.pass();
});
